//+------------------------------------------------------------------+
//|                                                 Level Sensor.mq4 |
//|                                          Copyright  2005, Sfen. |
//+------------------------------------------------------------------+
// modified by mladen - Oct 2013
// enhanced by fareastol - Oct 2013

#property indicator_chart_window

//---- input parameters
extern string TimeFrame = "";
extern int MAX_HISTORY=500;
extern double TargetH = 0.0;
extern double TargetL = 0.0;
extern double VAratio = 70.0;
extern bool ShowComment = False;

extern string ID = "LevSensor";
extern int Xshift = 50;
extern bool AsBackground = True;
extern bool ShowLevelByHiLo = False;
extern bool FixedRight = True;
extern color BarColor_OC = Khaki;
extern color BarColor_HL = DimGray;

int STEP = 1;
double PointStep;
int timeFrame;
//-------------------------------------------------
int init()
{
   if(TimeFrame=="") TimeFrame=Period();
   timeFrame = stringToTimeFrame(TimeFrame);
   PointStep = 1.0*Point*STEP;
   ID = ID+timeFrameToString(timeFrame);
   IndicatorShortName(timeFrameToString(timeFrame)+" LevelSensor");
   return(0);
}
//-------------------------------------------------
double MAX_OC(int i)
   {  return (MathMax(iOpen(NULL,timeFrame,i), iClose(NULL,timeFrame,i))); }
double MIN_OC(int i)
   {  return (MathMin(iOpen(NULL,timeFrame,i), iClose(NULL,timeFrame,i))); }
double MAX_HL(int i)
   {  return (MathMax(iHigh(NULL,timeFrame,i), iLow(NULL,timeFrame,i))); }
double MIN_HL(int i)
   {  return (MathMin(iHigh(NULL,timeFrame,i), iLow(NULL,timeFrame,i))); }
//-------------------------------------------------
void del_obj(string ID)
{
	for (int i = ObjectsTotal()-1; i>=0; i--)
   {
      string name = ObjectName(i);
      if (StringFind(name,ID)> -1) ObjectDelete(name);
   }     
   return(0);
}
//-------------------------------------------------
int deinit()
  {
   del_obj(ID);
   Comment("");
   return(0);
  }
//-------------------------------------------------
int start()
{
   del_obj(ID);

   double HH1 = 0;   double LL1 = 1000000;
   double HH2 = 0;   double LL2 = 1000000;

   int i, y, j, k;
   double price1, price2, TB_POC, TB_VAH=0, TB_VAL=0;
   int n1, n2, nPOC, nVAH, nVAL;
   int TB_HH, TB_LL;
   int TB_POCCount=0, TB_TotalCount=0, TB_VACount=0, VACount=0;
   int Count1[],Count2[],TBCount[];
   
   int History = MathMin(Bars,MAX_HISTORY);
   if(timeFrame!=Period()) History = MathMax(History,MAX_HISTORY*timeFrame/Period());
   
   for (i=1; i<History; i++)
   {
      y = iBarShift(NULL,timeFrame,Time[i]);
      HH1 = MathMax(HH1,MAX_OC(y));
      LL1 = MathMin(LL1,MIN_OC(y));
      HH2 = MathMax(HH2,MAX_HL(y));
      LL2 = MathMin(LL2,MIN_HL(y));
   }
   
   int Profile1 = (HH1-LL1)/PointStep+1;
   int Profile2 = (HH2-LL2)/PointStep+1;
   int Target_band = (TargetH-TargetL)/PointStep+1;
   ArrayResize(Count1,Profile1);  ArrayInitialize(Count1,0);
   ArrayResize(Count2,Profile2);  ArrayInitialize(Count2,0);
   
   for (i=1; i<History; i++)
   {
      y = iBarShift(NULL,timeFrame,Time[i]);
      for(price1=MIN_OC(y); price1<MAX_OC(y); price1 += PointStep)
      {
         n1 = (price1-LL1)/(PointStep);
         Count1[n1]++;    
      }
      if(ShowLevelByHiLo)
      for(price2=MIN_HL(y); price2<MAX_HL(y); price2 += PointStep)
      {
         n2 = (price2-LL2)/(PointStep);
         Count2[n2]++;    
      }
   }
   
   ArrayResize(TBCount,Target_band); ArrayInitialize(TBCount,0);
   TB_LL = (TargetL-LL1)/PointStep;
   for(j=0, n1=TB_LL; j<Target_band; j++, n1++)
   {
      TBCount[j] = Count1[n1];
      TB_TotalCount += TBCount[j];
   }

   nPOC = ArrayMaximum(TBCount);
   TB_POC = TargetL + nPOC*PointStep;
   TB_POCCount = TBCount[nPOC];
   TB_VACount = MathRound((VAratio/100)*TB_TotalCount);
   VACount = TB_POCCount;
   int upPOC = nPOC+1;
   int dnPOC = nPOC-1;

   while(VACount < TB_VACount)
   {
      if(upPOC < Target_band && (dnPOC==0 || TBCount[upPOC] >= TBCount[dnPOC]) )
      {
         VACount += TBCount[upPOC];
         upPOC ++;
      }
      else
      {
         VACount += TBCount[dnPOC];
         dnPOC --;
      }
   }
   
   TB_VAH = TargetL + upPOC*PointStep;
   TB_VAL = TargetL + dnPOC*PointStep;

   if(ShowComment) Comment("MaxHistory(bar) =",DoubleToStr(MAX_HISTORY,1),"\n","Profile(HL) = ",DoubleToStr(HH1,Digits),"-",DoubleToStr(LL1,Digits),"\n",
   "TargetBand(TB) = ",DoubleToStr(TargetH,Digits),"-",DoubleToStr(TargetL,Digits),"\n",
   "TB_TotalCount = ",DoubleToStr(TB_TotalCount,1)," --- ","TB_VACount = ",DoubleToStr(TB_VACount,1)," --- ","VACount = ",DoubleToStr(VACount,1),"\n",
   "nPOC = ",DoubleToStr(nPOC,1)," --- ","upPOC = ",DoubleToStr(upPOC,1)," --- ","dnPOC = ",DoubleToStr(dnPOC,1),"\n",
   "TB_VAH = ",DoubleToStr(TB_VAH, Digits)," --- ","TB_VAL = ",DoubleToStr(TB_VAL, Digits),"\n",
   "TB_POC = ",DoubleToStr(TB_POC, Digits),"\n","TB_POCCount = ",DoubleToStr(TB_POCCount,1));

   for (i=0; i<Profile1; i++)
   {
      double StartY = LL1 + PointStep*i;
      double EndY   = StartY;
      double StartX = Time[0] + Xshift*60*Period();
      datetime EndX = Time[0] + Xshift*60*Period();

      if(TimeDayOfWeek(Time[0])<=1)
      {  if (Count1[i]<=Xshift)  EndX -= Count1[i]*Period()*60;
         else                    EndX  = Time[Count1[i]-Xshift];
      }
      else if (FixedRight)       EndX  = Time[Count1[i]] + Xshift*Period()*60;
      else                       EndX  = EndX + Count1[i]*Period()*60;
      
      DrawLS(ID+" OC"+i, StartX, StartY, EndX, EndY, BarColor_OC);
   }
   
   if(ShowLevelByHiLo)
   {
      for (i=0; i<Profile2; i++)
      {
         double StartY2 = LL2 + PointStep*i;
         double EndY2   = StartY2;
         double StartX2 = Time[0] + Xshift*60*Period();
         datetime EndX2 = Time[0] + Xshift*60*Period();

         if(TimeDayOfWeek(Time[0])<=1)
         {  if (Count2[i]<=Xshift)  EndX2 -= Count2[i]*Period()*60;
            else                    EndX2  = Time[Count2[i]-Xshift];
         }
         else if (FixedRight)       EndX2  = Time[Count2[i]] + Xshift*Period()*60;
         else                       EndX2  = EndX2 + Count2[i]*Period()*60;
         
         DrawLS(ID+" HL"+i, StartX2, StartY2, EndX2, EndY2, BarColor_HL);
      }
   }
   return(0);
}
      
//+------------------------------------------------------------------+
void DrawLS(string name, int dt1, double p1, int dt2, double p2, color colorLS)
{
      string ObjName = name;
      ObjectDelete(ObjName);
      ObjectCreate(ObjName,OBJ_TREND,0,dt1,p1,dt2,p2);
      ObjectSet(ObjName,OBJPROP_RAY,0);
      ObjectSet(ObjName,OBJPROP_COLOR,colorLS);
      ObjectSet(ObjName,OBJPROP_BACK,AsBackground);
}
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int char = StringGetChar(s, length);
         if((char > 96 && char < 123) || (char > 223 && char < 256))
                     s = StringSetChar(s, length, char - 32);
         else if(char > -33 && char < 0)
                     s = StringSetChar(s, length, char + 224);
   }
   return(s);
}

